#include "ST7735.h"


void Tick(uint16_t i)
{
	while(--i);
}

void TFTIOInit()
{
	/* CLK   = Output
	 * DOUT  = Output
	 * CS    = Output
	 * RS    = Output
	 * BL    = Output
	 *
	 */

	sbi(TFT_CLK_DDR, TFT_CLK_PIN);
	sbi(TFT_DOUT_DDR, TFT_DOUT_PIN);
	sbi(TFT_CS_DDR, TFT_CS_PIN);
	sbi(TFT_RS_DDR, TFT_RS_PIN);
	sbi(TFT_BL_DDR, TFT_BL_PIN);
	
	sbi(TFT_CS_PORT, TFT_CS_PIN);
	cbi(TFT_CLK_PORT, TFT_CLK_PIN);
	cbi(TFT_DOUT_PORT, TFT_DOUT_PIN);
	cbi(TFT_RS_PORT, TFT_RS_PIN);
	cbi(TFT_BL_PORT, TFT_BL_PIN);
	
	//DDRD = 0;
	//PORTD = 0xFF;

	cbi(CENTER_DDR, CENTER_PIN);
	cbi(LEFT_DDR, LEFT_PIN);
	cbi(RIGHT_DDR, RIGHT_PIN);
	cbi(DOWN_DDR, DOWN_PIN);
	
	sbi(CENTER_PORT, CENTER_PIN);
	sbi(LEFT_PORT, LEFT_PIN);
	sbi(RIGHT_PORT, RIGHT_PIN);
	sbi(DOWN_PORT, DOWN_PIN);
	return;
}

void TFTBackLight(int state)
{
	if(state == 1) sbi(TFT_BL_PORT, TFT_BL_PIN);
	else cbi(TFT_BL_PORT, TFT_BL_PIN);
}

void TFTWriteCmd(uint8_t command)
{
	uint8_t i = 8;

	cbi(TFT_CS_PORT, TFT_CS_PIN);	/* Setup Chip Select */
	cbi(TFT_RS_PORT, TFT_RS_PIN);	/* R/S = 0 for Command */

	while(1) {
		i--;
		if tbi(command, i) sbi(TFT_DOUT_PORT, TFT_DOUT_PIN);
		else cbi(TFT_DOUT_PORT, TFT_DOUT_PIN);

		cbi(TFT_CLK_PORT, TFT_CLK_PIN);		/* Setup the Clock */
		sbi(TFT_CLK_PORT, TFT_CLK_PIN);		/* Reset the Clock */
		if(i == 0)break;
	}

	sbi(TFT_CS_PORT, TFT_CS_PIN);	/* Deselect Chip Select */

	return;
}

void TFTWriteData(uint8_t data)
{
	uint8_t i = 8;

	cbi(TFT_CS_PORT, TFT_CS_PIN);	/* Setup Chip Select */
	sbi(TFT_RS_PORT, TFT_RS_PIN);	/* R/S = 1 for Command */

	while(1) {
			i--;
			if tbi(data, i) sbi(TFT_DOUT_PORT, TFT_DOUT_PIN);
			else cbi(TFT_DOUT_PORT, TFT_DOUT_PIN);

			cbi(TFT_CLK_PORT, TFT_CLK_PIN);		/* Setup the Clock */
			sbi(TFT_CLK_PORT, TFT_CLK_PIN);		/* Reset the Clock */
			if(i == 0)break;
	}

	sbi(TFT_CS_PORT, TFT_CS_PIN);	/* Deselect Chip Select */

	return;
}

void TFTInit()
{

	TFTIOInit();

	TFTWriteCmd(ST7735_SWRESET); // software reset
		sbi(TFT_RESET_DDR, TFT_RESET_PIN);
	cbi(TFT_RESET_PORT, TFT_RESET_PIN);
	_delay_ms(500);
	sbi(TFT_RESET_PORT, TFT_RESET_PIN);
	_delay_ms(500);
	Tick(50);
	TFTWriteCmd(ST7735_SLPOUT);  // out of sleep mode
	Tick(500);

	TFTWriteCmd(ST7735_COLMOD);  // set color mode
	TFTWriteData(0x05);          // 16-bit color
	Tick(10);

	TFTWriteCmd(ST7735_FRMCTR1); // frame rate control
	TFTWriteData(0x00);          // fastest refresh
	TFTWriteData(0x06);          // 6 lines front porch
	TFTWriteData(0x03);          // 3 lines backporch
	Tick(10);

	TFTWriteCmd(ST7735_MADCTL);  // memory access control (directions)
	TFTWriteData(0xC8);          // row address/col address, bottom to top refresh

	TFTWriteCmd(ST7735_DISSET5); // display settings #5
	TFTWriteData(0x15);          // 1 clock cycle nonoverlap, 2 cycle gate rise, 3 cycle oscil. equalize
	TFTWriteData(0x02);          // fix on VTL

	TFTWriteCmd(ST7735_INVCTR);  // display inversion control
	TFTWriteData(0x0);           // line inversion
/*
	TFTWriteCmd(ST7735_PWCTR1);  // power control
	TFTWriteData(0x02);          // GVDD = 4.7V
	TFTWriteData(0x70);          // 1.0uA
	Tick(10);
	TFTWriteCmd(ST7735_PWCTR2);  // power control
	TFTWriteData(0x05);          // VGH = 14.7V, VGL = -7.35V
	TFTWriteCmd(ST7735_PWCTR3);  // power control
	TFTWriteData(0x01);          // Opamp current small
	TFTWriteData(0x02);          // Boost frequency


	TFTWriteCmd(ST7735_VMCTR1);  // power control
	TFTWriteData(0x3C);          // VCOMH = 4V
	TFTWriteData(0x38);          // VCOML = -1.1V
	Tick(10);

	TFTWriteCmd(ST7735_PWCTR6);  // power control
	TFTWriteData(0x11);
	TFTWriteData(0x15);
*/
	TFTWriteCmd(ST7735_GMCTRP1);
	TFTWriteData(0x09);
	TFTWriteData(0x16);
	TFTWriteData(0x09);
	TFTWriteData(0x20);
	TFTWriteData(0x21);
	TFTWriteData(0x1B);
	TFTWriteData(0x13);
	TFTWriteData(0x19);
	TFTWriteData(0x17);
	TFTWriteData(0x15);
	TFTWriteData(0x1E);
	TFTWriteData(0x2B);
	TFTWriteData(0x04);
	TFTWriteData(0x05);
	TFTWriteData(0x02);
	TFTWriteData(0x0E);
	TFTWriteCmd(ST7735_GMCTRN1);
	TFTWriteData(0x0B);
	TFTWriteData(0x14);
	TFTWriteData(0x08);
	TFTWriteData(0x1E);
	TFTWriteData(0x22);
	TFTWriteData(0x1D);
	TFTWriteData(0x18);
	TFTWriteData(0x1E);
	TFTWriteData(0x1B);
	TFTWriteData(0x1A);
	TFTWriteData(0x24);
	TFTWriteData(0x2B);
	TFTWriteData(0x06);
	TFTWriteData(0x06);
	TFTWriteData(0x02);
	TFTWriteData(0x0F);
	Tick(10);
/*
	TFTWriteCmd(ST7735_CASET);	// Set window area X
	TFTWriteData(0x00);
	TFTWriteData(0x02);
	TFTWriteData(0x00);
	TFTWriteData(0x81);

	TFTWriteCmd(ST7735_RASET);	// Set window area Y
	TFTWriteData(0x00);
	TFTWriteData(0x02);
	TFTWriteData(0x00);
	TFTWriteData(0x81);
*/
	TFTWriteCmd(ST7735_NORON);   // normal display on
	Tick(10);

	TFTWriteCmd(ST7735_DISPON);
	Tick(500);

	TFTBackLight(1);

}

void TFTSetWindow(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1)
{
	TFTWriteCmd(ST7735_CASET);   /* col address command */
	TFTWriteData(0x00);
	TFTWriteData(x0);          /* X Start */
	TFTWriteData(0x00);
	TFTWriteData(x1);          /* X end */

	TFTWriteCmd(ST7735_RASET);   /* row address command */
	TFTWriteData(0x00);
	TFTWriteData(y0);          /* Y Start */
	TFTWriteData(0x00);
	TFTWriteData(y1);          /* Y end */

}

void TFTPixel(uint16_t x, uint16_t y, uint16_t color)
{
	TFTSetWindow(x, y, x , y);
	TFTWriteCmd(ST7735_RAMWR);		/* RAM Access */
	TFTWriteData(color >> 8);
	TFTWriteData(color);

}

void TFTBackground(uint16_t xs, uint16_t ys, uint16_t xe, uint16_t ye, uint16_t color)
{
	TFTSetWindow(xs, ys, xe , ye);
	TFTWriteCmd(ST7735_RAMWR);		/* RAM Access */
	TFTWriteData(color >> 8);
	TFTWriteData(color);

}